#!/usr/bin/perl
#
$help = '
##############################################################################
## Script name: htmlcutter.pl
## Author: Mark Fyvie
## Version: 1.11
## Date: 11.03.2006
##
## Description: 
## This is a simple script which takes a source file, and cuts out marked
## chunks in the middle of the file, and dumps these chunks out as separate 
## files, while adding the contents of the beginning and end of the source 
## file as a header and footer in the destination files.
##
## For example: You write an HTML file with your standard header and nav bar 
## at the top of the file. You add your standard footer at the end of the file
## Now you put the contents of each of your separate pages one after another
## in the middle of the file. You can then view your entire set of pages on a 
## single page for editing, proof reading etc. Also for maintenance you can
## just maintain the single source file. When you are ready you can run this 
## script and it will produce all of the files you need.
##
## Tip: do not name your source file with the same name as one of the pages 
## within it. This will cause your source file to be overwritten. For safety
## add _source to the filename.
## 
## WARNING: This program expects all XHTML/HTML syntax to be in lower case!
##
## Use:
##
## htmlcutter.pl filename (where filename is the name of the source file)
## htmlcutter.pl --help (this file)
##
## This script works from a predefined set of rules. It expects that your sub-
## pages all begin with the following example tag:
##
## <!-- [HTMLCUTTER] filename= --> (note: this is case sensitive!)
##
## Therefore an example might be:
##
## <!-- [HTMLCUTTER] filename=index.html -->
##
## Before the footer a special tag must be used:
##
## <!-- [HTMLCUTTER] FOOTER -->
## 
## The following assumptions are made by the script:
##
## 1. Everything up to the first [HTMLCUTTER] tag is considered the header.
## (note comments are not permitted above the !DOCTYPE declaration in HTML!)
## 2. No end marker is required for each section, the [HTMLCUTTER] tag serves
## to notify the script that a new file is beginning, therefore the previous 
## file is assumed to have ended.
## 3. The footer begins only when the special keyword FOOTER is found, and ends
## when the end of the file is reached.
##
## Additional features:
##
## If the special keyword $currentDate (note: case sensitive!) is found within
## your source file it will be automatically converted to the current system
## date in the output file. This might be useful for putting a last updated
## date stamp into each footer.
##
## If the special keyword $root_ is found within your source file is will
## automatically be converted to the value you specify using the following
## command PRIOR to your [HTMLCUTTER] filename= command:
##
## <!-- [HTMLCUTTER] rootpath=../ -->
## <!-- [HTMLCUTTER] filename=somedir/somefile.html -->
##
## This will substitute any occurance of $root_ with ../ when outputting
## somefile.html, including the header and footer content. This allows you
## to place files in sub folders, but still have them reference files higher
## up in the tree. If no rootpath= command is given any $root_ keywords will
## be expanded to null values.
##
## Specifying different titles and meta keywords for different pages:
##
## If you wish to use different meta keywords, description and page titles 
## in different pages you must first set up a default set of values for 
## htmlcutter, then place the variables $metaKeywords, $metaDescription and 
## $pageTitle in place of the text you normally use. These then become the 
## defaults. See the following example:
## 
## <!-- [HTMLCUTTER-META] keywords=diving,photo album,exhibitplus,fyvie -->
##  <meta name="keywords" content="$metaKeywords" />
##
## <!-- [HTMLCUTTER-META] description=Provides a lot of good info on diving -->
##  <meta name="description" content="$metaDescription" />
##
## <!-- [HTMLCUTTER-TITLE] title=Welcome to fyvie.net -->
##  <title>$pageTitle</title>
##
## If you would like to change these values for a given page you must ensure
## that the additional [HTMLCUTTER] tags appear *before* the tag which 
## specifies the filename
##
## Please see the following example:
## <!-- [HTMLCUTTER] title=Diving resources at fyvie.net -->
## <!-- [HTMLCUTTER] keywords=tec diving, rebreathers, logbooks -->
## <!-- [HTMLCUTTER] filename=diving.html -->
##
## Including an external file in your output: 
##
## You may use the the include command to include an external file like this:
##
## <!-- [HTMLCUTTER-INCLUDE] filename=include.html -->
##
## This will include the entire contents of the file include.html into your
## output file, at the point at which you used this command. Please note that
## this command may not be used inside the header or footer, and the file
## itself may not contain additional htmlcutter internal commands (they can
## be used, but will not be processed if inside an include file).
##
##############################################################################\n\n';

$sourcefile = shift @ARGV;
if (!$sourcefile) {
  print "\nUsage: htmlcutter filename\n- where filename is the name of the source file to process.\n\nEnter htmlcutter --help for detailed instrutions\n\n";
exit
}
if ($sourcefile eq "--help") {
print $help;
exit

}

# Go fetch the current date and convert it into a variable which is remotely usable...
my ($dummy1, $dummy2, $dummy3, $day, $mon, $year, $dummy4, $dummy5, $dummy6 ) = localtime; 
if ($day < 10) {
$day = "0" . $day;
}
$mon ++;
if ($mon < 10) {
$mon = "0" . $mon;
}
$year = 1900 + $year;
$currentdate=  $day.".".$mon.".".$year;

# Pull in the header of the file, load it into array @htmlheader

open IN, $sourcefile
  or die "Can't open file $sourcefile for input";

my $position = "";

until ((eof IN) || ($position)) {
  $line = <IN>;

  # Load in a default title to use in case the title is not specified for individual files
  if (index($line, "[HTMLCUTTER-TITLE] title=") > -1) {
    $default_page_title = substr($line, (index($line, "title=") + 6), ((index($line, "-->") - 1) - (index($line, "title=") + 6)));
    $page_title = $default_page_title;
    $line = "";
  }

  # Load in a default meta keywords to use in case the meta keywords are not specified for individual files
  if (index($line, "[HTMLCUTTER-META] keywords=") > -1) {
    $default_meta_keywords = substr($line, (index($line, "keywords=") + 9), ((index($line, "-->") - 1) - (index($line, "keywords=") + 9)));
    $meta_keywords = $default_meta_keywords;
    $line = "";
  }

  # Load in a default meta description to use in case the meta description are not specified for individual files
  if (index($line, "[HTMLCUTTER-META] description=") > -1) {
    $default_meta_description = substr($line, (index($line, "description=") + 12), ((index($line, "-->") - 1) - (index($line, "description=") + 12)));
    $meta_description = $default_meta_description;
    $line = "";
  }


  if (index($line, "[HTMLCUTTER]") > -1) {
    $position = "header_end";
  } else {
    $_ = $line;
    s/\$currentDate/$currentdate/;
    s/\$pageTitle/$page_title/;
    s/\$metaKeywords/$meta_keywords/;
    s/\$metaDescription/$meta_description/;
    $line = $_;
    push @htmlheader, $line;
  }
}

close IN;

# Pull in the footer of the file, load it into array @htmlfooter

open IN, $sourcefile
  or die "Can't open file $sourcefile for input";

$position = "";

until (eof IN) {
  $line = <IN>;

  if (index($line, "[HTMLCUTTER] FOOTER") > -1) {
    $position = "footer_start"; 
    $line = "";
  }
  if ($position) {
    $_ = $line;
    s/\$currentDate/$currentdate/;
    $line = $_;
    push @htmlfooter, $line;
  }
}

# Load in the chunks, wrap them in a header and footer and spit out the correct filenames

close IN;

open IN, $sourcefile
  or die "Can't open file $sourcefile for input";

my $outfile = "";
my $include_file = "";
my $root_path = "";


until (eof IN) {
  $line = <IN>;

  # Check to see if a title has been specified, this must preceed the filename
  if (index($line, "[HTMLCUTTER] title=") > -1) {
    $page_title = substr($line, (index($line, "title=") + 6), ((index($line, "-->") - 1) - (index($line, "title=") + 6)));
    $line = "";
  }
  # Check to see if meta keywords have been specified, this must preceed the filename
  if (index($line, "[HTMLCUTTER] keywords=") > -1) {
    $meta_keywords = substr($line, (index($line, "keywords=") + 9), ((index($line, "-->") - 1) - (index($line, "keywords=") + 9)));
    $line = "";
  }
  # Check to see if meta description has been specified, this must preceed the filename
  if (index($line, "[HTMLCUTTER] description=") > -1) {
    $meta_description = substr($line, (index($line, "description=") + 12), ((index($line, "-->") - 1) - (index($line, "description=") + 12)));
    $line = "";
  }
  # Check to see if a path to the root of the site has been given, this must preceed the filename
  if (index($line, "[HTMLCUTTER] rootpath=") > -1) {
    $root_path = substr($line, (index($line, "rootpath=") + 9), ((index($line, "-->") - 1) - (index($line, "rootpath=") + 9)));
    $line = "";
  }
  
  # Check to see if another file should be included in the processing
  if (index($line, "[HTMLCUTTER-INCLUDE] filename=") > -1) {
    $include_file = substr($line, (index($line, "filename=") + 9), ((index($line, "-->") - 1) - (index($line, "filename=") + 9)));
    $line = "";

    if (($outfile eq $include_file) || ($include_file eq $sourcefile)) {
      print "\nException: You have attempted to use the include statement to include a file with the same name as your source file or one of your output files...\n\nSee documentation via htmlcutter.pl --help for more details.\n\n";
      $include_file = "";
    }
  }


  # Check to see if a filename is present

  if (index($line, "[HTMLCUTTER] filename=") > -1) {

    if ($outfile) {
      # Backup the array
      @htmlfooter2 = @htmlfooter;
      $_ = "@htmlfooter2";
      s/\$root_/$root_path_footer/g;
      @htmlfooter2 = $_;
      print OUT @htmlfooter2;
      close OUT;
      $root_path_footer = "";
      $root_path_body = "";
    }

    $outfile = substr($line, (index($line, "filename=") + 9), ((index($line, "-->") - 1) - (index($line, "filename=") + 9)));
    $line = "";  
  
    if ($outfile eq $sourcefile) {
      print "\nException: The source file contains a chunk with the same filename as itself. Have you renamed your source file to a different name? If not then you really should have...\n\nSee documentation via htmlcutter.pl --help for more details.\n\n";
      exit
    }

    open OUT, "> $outfile"
      or die "Can't open '$outfile' for output";

    # Note: The following chunks of code are not well written - I wish I'd thought this through properly...
    
    #these lines are needed in case we have null values for title, keywords or description

    $prepended_default_page_title = '<title>' . $default_page_title;
    $prepended_page_title = '<title>' . $page_title;
    $prepended_default_meta_keywords = '"keywords" content="' . $default_meta_keywords;
    $prepended_meta_keywords = '"keywords" content="' . $meta_keywords;
    $prepended_default_meta_description = '"description" content="' . $default_meta_description;
    $prepended_meta_description = '"description" content="' . $meta_description;
    
    # Replace items in the header as required
    $_ = "@htmlheader";
    s/$prepended_default_page_title/$prepended_page_title/;
    s/$prepended_default_meta_keywords/$prepended_meta_keywords/;	
    s/$prepended_default_meta_description/$prepended_meta_description/;	
    @htmlheader = $_;
    
    # Backup the array
    @htmlheader2 = @htmlheader;
    $_ = "@htmlheader2";
    s/\$root_/$root_path/g;
    @htmlheader2 = $_;
    print OUT @htmlheader2;
    $root_path_footer = $root_path;
    $root_path_body = $root_path;
    $root_path = "";
    
    # Reset the title and metadata to default for the next output file

    $_ = "@htmlheader";
    s/$prepended_page_title/$prepended_default_page_title/;
    s/$prepended_meta_keywords/$prepended_default_meta_keywords/;	
    s/$prepended_meta_description/$prepended_default_meta_description/;	
   
    @htmlheader = $_;
    $page_title = $default_page_title;
    $meta_keywords = $default_meta_keywords;	
    $meta_description = $default_meta_description;	

  }

  if (index($line, "[HTMLCUTTER] FOOTER") > -1) {
    $footer_reached="true";
  }

  if (($outfile) && (!$footer_reached)) {
    if ($include_file ne "") {
      if (open INCLUDE, $include_file) {
        until (eof INCLUDE) {
          $line = <INCLUDE>;
          $_ = $line;
          s/\$currentDate/$currentdate/;
          s/\$root_/$root_path_body/g;
          $line = $_;
          print OUT $line;
        }  
        close INCLUDE;
      } else {
        print "Warning: Couldn't open include file $include_file\n";
      }  
      $include_file = "";
    }

    $_ = $line;
    s/\$currentDate/$currentdate/;
    s/\$root_/$root_path_body/g;
    $line = $_;
    print OUT $line;
  }

}

# We've reached the end of the file, so we have to clean up the stuff we missed in the until loop above
close IN;
$_ = "@htmlfooter";
s/\$root_/$root_path_footer/g;
@htmlfooter = $_;
print OUT @htmlfooter;
close OUT;

# We're done, all filenames have been output
